/*
    plugin.cpp  -  Floodcast plugin

    Copyright (c) 2002 by Ryan Cumming
    Copyright (c) 2004 by Stefan Gehn <metz AT gehn.net>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <kaction.h>
#include <kdebug.h>
#include <klocale.h>

#include <noatun/app.h>
#include <noatun/player.h>
#include <noatun/playlist.h>
#include <noatun/stdaction.h>

#include "status.h"
#include "server.h"
#include "plugin.h"
#include "prefs.h"
#include "mp3stream.h"
#include "icecastpipe.h"
#include "floodcastconfig.h"

FloodcastPlugin::FloodcastPlugin()
{
	kdDebug(66666) << k_funcinfo << endl;
	mServer = 0;
	mPipe = 0;
	mPrefs = 0;
	mStatusWindow = 0;
	mStatusAction = 0;
}


FloodcastPlugin::~FloodcastPlugin()
{
	kdDebug(66666) << k_funcinfo << endl;
	napp->pluginActionMenu()->remove(mUsePlugin);
	napp->pluginActionMenu()->remove(mStatusAction);
}


void FloodcastPlugin::init()
{
	kdDebug(66666) << k_funcinfo << endl;

	mPrefs = new Prefs(this);
	connect(mPrefs, SIGNAL(saved()), this, SLOT(slotApplyConfig()));

	mStatusWindow = new FloodcastStatus();

	mStatusAction = new KAction(i18n("&Floodcast Server Status..."), "info", 0,
		this, SLOT(slotShowStatusWindow()), this, "mStatusAction");
	mUsePlugin = new KToggleAction(i18n("Use Floodcast"), 0,
		this, SLOT(slotEnableStream()), this, "mUsePlugin");

	napp->pluginActionMenu()->insert(mStatusAction);
	napp->pluginActionMenu()->insert(mUsePlugin);

	connect(napp->player(), SIGNAL(timeout()), this, SLOT(slotRefreshTitle()));

	slotApplyConfig();
}


void FloodcastPlugin::slotApplyConfig()
{
	kdDebug(66666) << k_funcinfo << endl;

	if (FloodcastConfig::mode() == FloodcastConfig::Standalone)
	{
		kdDebug(66666) << k_funcinfo << "Standalone Mode" << endl;

		delete mPipe;
		mPipe = 0;
		if(mServer)
		{
			mServer->setListenPort(FloodcastConfig::serverPort());
			mServer->setMaxUsers(FloodcastConfig::serverMaxUsers());
			mServer->restartEncoder();
		}
		else
		{
			mServer = new IcecastServer(this, "icecastserver");
			mUsePlugin->setChecked(false);
			mStatusAction->setEnabled(true);
			mStatusWindow->setServer(mServer);
			connect(mServer, SIGNAL(statusChange(bool)),
				mUsePlugin, SLOT(setChecked(bool)));
		}
	}
	else
	{
		kdDebug(66666) << k_funcinfo << "Forward Mode" << endl;

		delete mServer;
		mServer = 0;
		if (mPipe)
		{
			if(mUsePlugin->isChecked())
				mPipe->restart();
		}
		else
		{
			mPipe = new IcecastPipe(this, "icecastpipe");
			mUsePlugin->setChecked(false);
			mStatusAction->setEnabled(false);
			mStatusWindow->hide(); // make sure the server's status-window is gone
			connect(mPipe, SIGNAL(statusChange(bool)), mUsePlugin, SLOT(setChecked(bool)));
		}
	}
}


void FloodcastPlugin::slotEnableStream()
{
	kdDebug(66666) << k_funcinfo << "checked = " << mUsePlugin->isChecked() << endl;
	if(mUsePlugin->isChecked())
	{
		if (mPipe)
		{
			slotRefreshTitle();
			mPipe->start();
		}
		else if (mServer)
		{
			mServer->start();
		}
	}
	else
	{
		if (mPipe)
			mPipe->stop();
		else if (mServer)
			mServer->stop();
	}
}


void FloodcastPlugin::slotShowStatusWindow()
{
	if (mStatusWindow->isHidden())
		mStatusWindow->show();
	else
		mStatusWindow->raise();
}


void FloodcastPlugin::slotRefreshTitle()
{
	//kdDebug(66666) << k_funcinfo << endl;
	if (!mUsePlugin->isChecked())
		return;

	if (FloodcastConfig::fixedTrackname())
	{
		if (mPipe)
			mPipe->setTitle(FloodcastConfig::streamTrackname());
		//else if (mServer)
	}
	else
	{
		PlaylistItem cur = napp->player()->current();
		if(!cur.isNull())
		{
			QString title = "";
			QString artist = cur.property("author");
			QString track = cur.property("title");
			if (!artist.isEmpty())
			{
				if (!track.isEmpty())
					title = i18n("%1 - %2").arg(artist, track);
				else
					title = artist;
			}
			else
			{
				if (!track.isEmpty())
					title = track;
			}

			if (mPipe)
				mPipe->setTitle(title);
			//else if (mServer)
		}
	}
}

#include "plugin.moc"
